/* In the line search for first iteration, there is very little information
   available for choosing a suitable stepsize. By default, the code employs
   very low order approximations to the function to estimate a suitable
   stepsize. In some cases, this initial stepsize can be problematic.
   For example, if the cost function contains a log function, the initial
   step might cause the code to try to compute the log of a negative number.
   If the cost function contains an exponential, then the initial stepsize
   might lead to an overflow. In either case, NaNs are potentially generated.
   To assist the code by providing an initial stepsize, store the initial
   step for the line search in the parameter ``step''.
   In the following example, the initial stepsize is set to 1.
   The output is below:

   Termination status: 0
   Convergence tolerance for gradient satisfied
   absolute largest component of gradient: 6.415111e-09
   function value: -6.530787e+02
   cg iterations: 31
   function evaluations: 52
   gradient evaluations: 43 */

#include <math.h>
#include "cg_user.h"

double myvalue
(
    double   *x ,
    INT       n
) ;

void mygrad
(
    double    *g ,
    double    *x ,
    INT        n
) ;

int main (void)
{
    double *x, *work ;
    INT i, mydim ;
    int status ;
    cg_stats Stats ;
    cg_parameter Parm ;

    cg_default (&Parm) ;
    Parm.step = 1. ;

/* allocate work space */
    mydim = 100 ;
    x = (double *) malloc (mydim*sizeof (double)) ;
    work = (double *) malloc (4*mydim*sizeof (double)) ;

/* starting guess */
    for (i = 0; i < mydim; i++) x [i] = 1. ;

/* solve the problem */
    status = cg_descent (x, mydim, &Stats, &Parm, 1.e-8, myvalue, mygrad, work);

/* free work space */
    free (x) ;
    free (work) ;
}

double myvalue
(
    double   *x ,
    INT       n
)
{
    double f, t ;
    INT i ;
    f = 0. ;
    for (i = 0; i < n; i++)
    {
        t = i+1 ;
        t = sqrt (t) ;
        f += exp (x [i]) - t*x [i] ;
    }
    return (f) ;
}

void mygrad
(
    double    *g ,
    double    *x ,
    INT        n
)
{
    double t ;
    INT i ;
    for (i = 0; i < n; i++)
    {
        t = i + 1 ;
        t = sqrt (t) ;
        g [i] = exp (x [i]) -  t ;
    }
    return ;
}
