/* After the first iteration, the line search might take too large a
   step, when it tries to bracket a local minimizer along the search
   direction. In this expansion phase, the expansion factor is rho.
   The default value of rho is 5. Thus, in each step of the expansion
   phase, the bracketing interval grows by a factor of 5. If the cost
   function contains a log or an exponential function, this
   expansion might result in either an overflow or an attempt to compute
   the log of a negative number. A less aggressive expansion may avoid
   this problem. In the following example, we choose a small initial
   stepsize (initial step is 1.e-5), QuadStep is FALSE, and rho is 1.5.
   The code has to do a number of expansions to reach a suitable
   interval bracketing the minimizer in the initial search direction.

 
    Termination status: 0
    Convergence tolerance for gradient satisfied
    absolute largest component of gradient: 5.740862e-09
    function value: -6.530787e+02
    cg iterations: 34
    function evaluations: 58
    gradient evaluations: 92

   We then set rho = 5. In this case, fewer expansions are needed to bracket
   the minimizer in the initial search direction:

    Termination status: 0
    Convergence tolerance for gradient satisfied
    absolute largest component of gradient: 5.542049e-09
    function value: -6.530787e+02
    cg iterations: 35
    function evaluations: 42
    gradient evaluations: 77 */

#include <math.h>
#include "cg_user.h"

double myvalue
(
    double   *x ,
    INT       n
) ;

void mygrad
(
    double    *g ,
    double    *x ,
    INT        n
) ;

int main (void)
{
    double *x, *work ;
    INT i, mydim ;
    int status ;
    cg_stats Stats ;
    cg_parameter Parm ;

/* set parameter values */
    cg_default (&Parm) ;
    Parm.step = 1.e-5 ;
    Parm.QuadStep = FALSE ;
    Parm.rho = 1.5 ;

/* allocate work space */
    mydim = 100 ;
    x = (double *) malloc (mydim*sizeof (double)) ;
    work = (double *) malloc (4*mydim*sizeof (double)) ;

/* starting guess */
    for (i = 0; i < mydim; i++) x [i] = 1. ;

/* solve the problem */
    status = cg_descent (x, mydim, &Stats, &Parm, 1.e-8, myvalue, mygrad, work);

/* free work space */
    free (x) ;
    free (work) ;
}

double myvalue
(
    double   *x ,
    INT       n
)
{
    double f, t ;
    INT i ;
    f = 0. ;
    for (i = 0; i < n; i++)
    {
        t = i+1 ;
        t = sqrt (t) ;
        f += exp (x [i]) - t*x [i] ;
    }
    return (f) ;
}

void mygrad
(
    double    *g ,
    double    *x ,
    INT        n
)
{
    double t ;
    INT i ;
    for (i = 0; i < n; i++)
    {
        t = i + 1 ;
        t = sqrt (t) ;
        g [i] = exp (x [i]) -  t ;
    }
    return ;
}
